@@ -35,6 +35,9 @@ try:
 except NameError:
     unicode = str
 
+from locale import atof, setlocale, LC_ALL
+
+setlocale(LC_ALL, '')
 
 def write_xml(obj, filepath=None, pretty=False):
     tree = etree.ElementTree(obj._elem)
@@ -92,9 +95,7 @@ class IntAttr(Attr):
 
     def __get__(self, instance, cls):
         result = super(IntAttr, self).__get__(instance, cls)
-        if result is None and (
-            isinstance(instance, JUnitXml) or isinstance(instance, TestSuite)
-        ):
+        if result is None and isinstance(instance, (JUnitXml, TestSuite)):
             instance.update_statistics()
             result = super(IntAttr, self).__get__(instance, cls)
         return int(result) if result else None
@@ -114,12 +115,10 @@ class FloatAttr(Attr):
 
     def __get__(self, instance, cls):
         result = super(FloatAttr, self).__get__(instance, cls)
-        if result is None and (
-            isinstance(instance, JUnitXml) or isinstance(instance, TestSuite)
-        ):
+        if result is None and isinstance(instance, (JUnitXml, TestSuite)):
             instance.update_statistics()
             result = super(FloatAttr, self).__get__(instance, cls)
-        return float(result) if result else None
+        return atof(result) if result else None
 
     def __set__(self, instance, value):
         if not (isinstance(value, float) or isinstance(value, int)):
@@ -161,8 +160,8 @@ class Element(with_metaclass(junitxml, object)):
                 ['%s="%s"' % (key, self._elem.attrib[key]) for key in keys]
             )
             return """<Element '%s' %s>""" % (tag, attrs_str)
-        else:
-            return """<Element '%s'>""" % tag
+
+        return """<Element '%s'>""" % tag
 
     def append(self, sub_elem):
         """Adds the element subelement to the end of this elements internal
@@ -408,12 +407,12 @@ class TestSuite(Element):
                 self.add_testsuite(suite)
             self.update_statistics()
             return self
-        else:
-            result = JUnitXml()
-            result.filepath = self.filepath
-            result.add_testsuite(self)
-            result.add_testsuite(other)
-            return result
+
+        result = JUnitXml()
+        result.filepath = self.filepath
+        result.add_testsuite(self)
+        result.add_testsuite(other)
+        return result
 
     def remove_testcase(self, testcase):
         """Removes a test case from the suite."""
@@ -428,14 +427,15 @@ class TestSuite(Element):
         time = 0
         for case in self:
             tests += 1
-            if isinstance(case.result, Failure):
-                failures += 1
-            elif isinstance(case.result, Error):
-                errors += 1
-            elif isinstance(case.result, Skipped):
-                skipped += 1
             if case.time is not None:
                 time += case.time
+            for entry in case.result:
+                if isinstance(entry, Failure):
+                    failures += 1
+                elif isinstance(entry, Error):
+                    errors += 1
+                elif isinstance(entry, Skipped):
+                    skipped += 1
         self.tests = tests
         self.errors = errors
         self.failures = failures
@@ -583,6 +583,13 @@ class Result(Element):
             and self.message == other.message
         )
 
+    @property
+    def text(self):
+        return self._elem.text
+
+    @text.setter
+    def text(self, value):
+        self._elem.text = value
 
 class Skipped(Result):
     """Test result when the case is skipped."""
@@ -633,46 +640,48 @@ class TestCase(Element):
     classname = Attr()
     time = FloatAttr()
 
-    def __init__(self, name=None):
+    def __init__(self, name=None, classname=None, time=None):
         super(TestCase, self).__init__(self._tag)
-        self.name = name
+        if name is not None:
+            self.name = name
+        if classname is not None:
+            self.classname = classname
+        if time is not None:
+            self.time = float(time)
 
     def __hash__(self):
         return super(TestCase, self).__hash__()
 
+    def __iter__(self):
+        all_types = set.union(POSSIBLE_RESULTS, {SystemOut}, {SystemErr})
+        for elem in self._elem.iter():
+            for entry_type in all_types:
+                if elem.tag == entry_type._tag:
+                    yield entry_type.fromelem(elem)
+
     def __eq__(self, other):
         # TODO: May not work correctly if unreliable hash method is used.
         return hash(self) == hash(other)
 
     @property
     def result(self):
-        """One of the Failure, Skipped, or Error objects."""
+        """A list of Failure, Skipped, or Error objects."""
         results = []
-        for res in POSSIBLE_RESULTS:
-            result = self.child(res)
-            if result is not None:
-                results.append(result)
-        if len(results) > 1:
-            raise JUnitXmlError("Only one result allowed per test case.")
-        elif len(results) == 0:
-            return None
-        else:
-            return results[0]
+        for entry in self:
+            if isinstance(entry, tuple(POSSIBLE_RESULTS)):
+                results.append(entry)
+
+        return results
 
     @result.setter
     def result(self, value):
         # First remove all existing results
-        for res in POSSIBLE_RESULTS:
-            result = self.child(res)
-            if result is not None:
-                self.remove(result)
-        # Then add current result
-        if (
-            isinstance(value, Skipped)
-            or isinstance(value, Failure)
-            or isinstance(value, Error)
-        ):
-            self.append(value)
+        for entry in self:
+            if any(isinstance(entry, r) for r in POSSIBLE_RESULTS ):
+                self.remove(entry)
+        for entry in value:
+            if any(isinstance(entry, r) for r in POSSIBLE_RESULTS ):
+                self.append(entry)
 
     @property
     def system_out(self):
